@@ -1,4 +1,5 @@
-// Copyright (C) 2009 NICTA
+// Copyright (C) 2010 NICTA and the authors listed below
+// http://nicta.com.au
 // 
 // Authors:
 // - Conrad Sanderson (conradsand at ieee dot org)
@@ -23,8 +24,8 @@ class Col : public Mat<eT>, public BaseVec< eT, Col<eT> >
   {
   public:
   
-  typedef eT elem_type;
-  typedef typename get_pod_type<elem_type>::pod_type pod_type;
+  typedef eT                                elem_type;
+  typedef typename get_pod_type<eT>::result pod_type;
   
   
   inline                     Col();
@@ -75,10 +76,26 @@ class Col : public Mat<eT>, public BaseVec< eT, Col<eT> >
   template<typename T1, typename op_type> inline const Col&  operator=(const Op<T1, op_type>& X);
   template<typename T1, typename op_type> inline const Col& operator*=(const Op<T1, op_type>& X);
   
+  template<typename T1, typename eop_type> inline                   Col(const eOp<T1, eop_type>& X);
+  template<typename T1, typename eop_type> inline const Col&  operator=(const eOp<T1, eop_type>& X);
+  template<typename T1, typename eop_type> inline const Col& operator*=(const eOp<T1, eop_type>& X);
+  
+  template<typename T1, typename op_type> inline                   Col(const mtOp<eT, T1, op_type>& X);
+  template<typename T1, typename op_type> inline const Col&  operator=(const mtOp<eT, T1, op_type>& X);
+  template<typename T1, typename op_type> inline const Col& operator*=(const mtOp<eT, T1, op_type>& X);
+  
   template<typename T1, typename T2, typename glue_type> inline                   Col(const Glue<T1, T2, glue_type>& X);
   template<typename T1, typename T2, typename glue_type> inline const Col&  operator=(const Glue<T1, T2, glue_type>& X);
   template<typename T1, typename T2, typename glue_type> inline const Col& operator*=(const Glue<T1, T2, glue_type>& X);
-    
+  
+  template<typename T1, typename T2, typename eglue_type> inline                   Col(const eGlue<T1, T2, eglue_type>& X);
+  template<typename T1, typename T2, typename eglue_type> inline const Col&  operator=(const eGlue<T1, T2, eglue_type>& X);
+  template<typename T1, typename T2, typename eglue_type> inline const Col& operator*=(const eGlue<T1, T2, eglue_type>& X);
+  
+  template<typename T1, typename T2, typename glue_type> inline                   Col(const mtGlue<eT, T1, T2, glue_type>& X);
+  template<typename T1, typename T2, typename glue_type> inline const Col&  operator=(const mtGlue<eT, T1, T2, glue_type>& X);
+  template<typename T1, typename T2, typename glue_type> inline const Col& operator*=(const mtGlue<eT, T1, T2, glue_type>& X);
+  
   inline void  set_size(const u32 n_elem);
   inline void  set_size(const u32 n_rows, const u32 n_cols);
   
@@ -93,8 +110,24 @@ class Col : public Mat<eT>, public BaseVec< eT, Col<eT> >
   inline void ones(const u32 n_elem);
   inline void ones(const u32 n_rows, const u32 n_cols);
   
-  inline void load(const std::string name, const file_type type = auto_detect);
+  
+  inline void load(const std::string   name, const file_type type = auto_detect, const bool print_status = true);
+  inline void load(      std::istream& is,   const file_type type = auto_detect, const bool print_status = true);
+  
+  inline void quiet_load(const std::string   name, const file_type type = auto_detect);
+  inline void quiet_load(      std::istream& is,   const file_type type = auto_detect);
+  
+  
+  typedef       eT*       row_iterator;
+  typedef const eT* const_row_iterator;
+  
+  inline       row_iterator begin_row(const u32 row_num);
+  inline const_row_iterator begin_row(const u32 row_num) const;
+  
+  inline       row_iterator end_row  (const u32 row_num);
+  inline const_row_iterator end_row  (const u32 row_num) const;
   };
 
 
+
 //! @}
