@@ -1,4 +1,5 @@
-// Copyright (C) 2009 NICTA
+// Copyright (C) 2010 NICTA and the authors listed below
+// http://nicta.com.au
 // 
 // Authors:
 // - Conrad Sanderson (conradsand at ieee dot org)
@@ -318,9 +319,20 @@ Mat<eT>::operator+=(const eT val)
   {
   arma_extra_debug_sigprint();
   
-  for(u32 i=0; i<n_elem; ++i)
+        eT* local_ptr    = memptr();
+  const u32 local_n_elem = n_elem;
+    
+  u32 i,j;
+  
+  for(i=0, j=1; j<local_n_elem; i+=2, j+=2)
     {
-    access::rw(mem[i]) += val;
+    local_ptr[i] += val;
+    local_ptr[j] += val;
+    }
+  
+  if(i < local_n_elem)
+    {
+    local_ptr[i] += val;
     }
   
   return *this;
@@ -336,11 +348,22 @@ Mat<eT>::operator-=(const eT val)
   {
   arma_extra_debug_sigprint();
   
-  for(u32 i=0; i<n_elem; ++i)
+        eT* local_ptr    = memptr();
+  const u32 local_n_elem = n_elem;
+    
+  u32 i,j;
+  
+  for(i=0, j=1; j<local_n_elem; i+=2, j+=2)
     {
-    access::rw(mem[i]) -= val;
+    local_ptr[i] -= val;
+    local_ptr[j] -= val;
     }
-      
+  
+  if(i < local_n_elem)
+    {
+    local_ptr[i] -= val;
+    }
+  
   return *this;
   }
 
@@ -354,9 +377,20 @@ Mat<eT>::operator*=(const eT val)
   {
   arma_extra_debug_sigprint();
   
-  for(u32 i=0; i<n_elem; ++i)
+        eT* local_ptr    = memptr();
+  const u32 local_n_elem = n_elem;
+    
+  u32 i,j;
+  
+  for(i=0, j=1; j<local_n_elem; i+=2, j+=2)
     {
-    access::rw(mem[i]) *= val;
+    local_ptr[i] *= val;
+    local_ptr[j] *= val;
+    }
+  
+  if(i < local_n_elem)
+    {
+    local_ptr[i] *= val;
     }
   
   return *this;
@@ -372,9 +406,20 @@ Mat<eT>::operator/=(const eT val)
   {
   arma_extra_debug_sigprint();
   
-  for(u32 i=0; i<n_elem; ++i)
+        eT* local_ptr    = memptr();
+  const u32 local_n_elem = n_elem;
+    
+  u32 i,j;
+  
+  for(i=0, j=1; j<local_n_elem; i+=2, j+=2)
     {
-    access::rw(mem[i]) /= val;
+    local_ptr[i] /= val;
+    local_ptr[j] /= val;
+    }
+  
+  if(i < local_n_elem)
+    {
+    local_ptr[i] /= val;
     }
   
   return *this;
@@ -502,7 +547,26 @@ Mat<eT>::operator+=(const Mat<eT>& m)
   {
   arma_extra_debug_sigprint();
   
-  glue_plus::apply_inplace(*this, m);
+  arma_debug_assert_same_size(*this, m, "matrix addition");
+  
+  const u32 local_n_elem = m.n_elem;
+  
+        eT* out_mem = (*this).memptr();
+  const eT* m_mem   = m.memptr();
+  
+  u32 i,j;
+  
+  for(i=0, j=1; j<local_n_elem; i+=2, j+=2)
+    {
+    out_mem[i] += m_mem[i];
+    out_mem[j] += m_mem[j];
+    }
+  
+  if(i < local_n_elem)
+    {
+    out_mem[i] += m_mem[i];
+    }
+  
   return *this;
   }
 
@@ -516,7 +580,26 @@ Mat<eT>::operator-=(const Mat<eT>& m)
   {
   arma_extra_debug_sigprint();
   
-  glue_minus::apply_inplace(*this, m);
+  arma_debug_assert_same_size(*this, m, "matrix subtraction");
+  
+  const u32 local_n_elem = m.n_elem;
+  
+        eT* out_mem = (*this).memptr();
+  const eT* m_mem   = m.memptr();
+  
+  u32 i,j;
+  
+  for(i=0, j=1; j<local_n_elem; i+=2, j+=2)
+    {
+    out_mem[i] -= m_mem[i];
+    out_mem[j] -= m_mem[j];
+    }
+  
+  if(i < local_n_elem)
+    {
+    out_mem[i] -= m_mem[i];
+    }
+  
   return *this;
   }
 
@@ -544,7 +627,26 @@ Mat<eT>::operator%=(const Mat<eT>& m)
   {
   arma_extra_debug_sigprint();
   
-  glue_schur::apply_inplace(*this, m);
+  arma_debug_assert_same_size(*this, m, "element-wise matrix multplication");
+  
+  const u32 local_n_elem = m.n_elem;
+  
+        eT* out_mem = (*this).memptr();
+  const eT* m_mem   = m.memptr();
+  
+  u32 i,j;
+  
+  for(i=0, j=1; j<local_n_elem; i+=2, j+=2)
+    {
+    out_mem[i] *= m_mem[i];
+    out_mem[j] *= m_mem[j];
+    }
+  
+  if(i < local_n_elem)
+    {
+    out_mem[i] *= m_mem[i];
+    }
+  
   return *this;
   }
 
@@ -558,7 +660,26 @@ Mat<eT>::operator/=(const Mat<eT>& m)
   {
   arma_extra_debug_sigprint();
   
-  glue_div::apply_inplace(*this, m);
+  arma_debug_assert_same_size(*this, m, "element-wise matrix division");
+  
+  const u32 local_n_elem = m.n_elem;
+  
+        eT* out_mem = (*this).memptr();
+  const eT* m_mem   = m.memptr();
+  
+  u32 i,j;
+  
+  for(i=0, j=1; j<local_n_elem; i+=2, j+=2)
+    {
+    out_mem[i] /= m_mem[i];
+    out_mem[j] /= m_mem[j];
+    }
+  
+  if(i < local_n_elem)
+    {
+    out_mem[i] /= m_mem[i];
+    }
+  
   return *this;
   }
 
@@ -676,7 +797,7 @@ Mat<eT>::operator*=(const subview<eT>& X)
   {
   arma_extra_debug_sigprint();
   
-  subview<eT>::times_inplace(*this, X);
+  glue_times::apply_inplace(*this, X);
   return *this;
   }
 
@@ -845,6 +966,75 @@ Mat<eT>::operator=(const diagview<eT>& X)
 
 
 
+//! in-place matrix addition (using a diagview on the right-hand-side)
+template<typename eT>
+inline
+const Mat<eT>&
+Mat<eT>::operator+=(const diagview<eT>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  diagview<eT>::plus_inplace(*this, X);
+  return *this;
+  }
+
+
+//! in-place matrix subtraction (using a diagview on the right-hand-side)
+template<typename eT>
+inline
+const Mat<eT>&
+Mat<eT>::operator-=(const diagview<eT>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  diagview<eT>::minus_inplace(*this, X);
+  return *this;
+  }
+
+
+
+//! in-place matrix mutiplication (using a diagview on the right-hand-side)
+template<typename eT>
+inline
+const Mat<eT>&
+Mat<eT>::operator*=(const diagview<eT>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  glue_times::apply_inplace(*this, X);
+  return *this;
+  }
+
+
+
+//! in-place element-wise matrix mutiplication (using a diagview on the right-hand-side)
+template<typename eT>
+inline
+const Mat<eT>&
+Mat<eT>::operator%=(const diagview<eT>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  diagview<eT>::schur_inplace(*this, X);
+  return *this;
+  }
+
+
+
+//! in-place element-wise matrix division (using a diagview on the right-hand-side)
+template<typename eT>
+inline
+const Mat<eT>&
+Mat<eT>::operator/=(const diagview<eT>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  diagview<eT>::div_inplace(*this, X);
+  return *this;
+  }
+
+
+
 //! creation of subview (row vector)
 template<typename eT>
 arma_inline
@@ -1033,7 +1223,7 @@ Mat<eT>::diag(const s32 in_id)
   arma_debug_check
     (
     (row_offset >= n_rows) || (col_offset >= n_cols),
-    "Mat::diag(): out of bounds"
+    "Mat::diag(): requested diagonal out of bounds"
     );
   
   const u32 len = (std::min)(n_rows - row_offset, n_cols - col_offset);
@@ -1057,7 +1247,7 @@ Mat<eT>::diag(const s32 in_id) const
   arma_debug_check
     (
     (row_offset >= n_rows) || (col_offset >= n_cols),
-    "Mat::diag(): out of bounds"
+    "Mat::diag(): requested diagonal out of bounds"
     );
   
   
@@ -1173,9 +1363,9 @@ Mat<eT>::operator+=(const Op<T1, op_type>& X)
   
   isnt_same_type<eT, typename T1::elem_type>::check();
   
-  glue_plus::apply_inplace(*this, X);
+  const Mat<eT> m(X);
   
-  return *this;
+  return (*this).operator+=(m);
   }
 
 
@@ -1191,9 +1381,9 @@ Mat<eT>::operator-=(const Op<T1, op_type>& X)
   
   isnt_same_type<eT, typename T1::elem_type>::check();
   
-  glue_minus::apply_inplace(*this, X);
+  const Mat<eT> m(X);
   
-  return *this;
+  return (*this).operator-=(m);
   }
 
 
@@ -1226,9 +1416,10 @@ Mat<eT>::operator%=(const Op<T1, op_type>& X)
   arma_extra_debug_sigprint();
   
   isnt_same_type<eT, typename T1::elem_type>::check();
-  glue_schur::apply_inplace(*this, X);
   
-  return *this;
+  const Mat<eT> m(X);
+  
+  return (*this).operator%=(m);
   }
 
 
@@ -1243,13 +1434,253 @@ Mat<eT>::operator/=(const Op<T1, op_type>& X)
   arma_extra_debug_sigprint();
   
   isnt_same_type<eT, typename T1::elem_type>::check();
-  glue_div::apply_inplace(*this, X);
+  
+  const Mat<eT> m(X);
+  
+  return (*this).operator/=(m);
+  }
+
+
+
+//! create a matrix from eOp, i.e. run the previously delayed unary operations
+template<typename eT>
+template<typename T1, typename eop_type>
+inline
+Mat<eT>::Mat(const eOp<T1, eop_type>& X)
+  : n_rows(0)
+  , n_cols(0)
+  , n_elem(0)
+  , use_aux_mem(false)
+  //, mem(0)
+  , mem(mem)
+  {
+  arma_extra_debug_sigprint_this(this);
+
+  isnt_same_type<eT, typename T1::elem_type>::check();
+  
+  eop_type::apply(*this, X);
+  }
+
+
+
+//! create a matrix from eOp, i.e. run the previously delayed unary operations
+template<typename eT>
+template<typename T1, typename eop_type>
+inline
+const Mat<eT>&
+Mat<eT>::operator=(const eOp<T1, eop_type>& X)
+  {
+  arma_extra_debug_sigprint();
+
+  isnt_same_type<eT, typename T1::elem_type>::check();
+  
+  eop_type::apply(*this, X);
   
   return *this;
   }
 
 
 
+template<typename eT>
+template<typename T1, typename eop_type>
+inline
+const Mat<eT>&
+Mat<eT>::operator+=(const eOp<T1, eop_type>& X)
+  {
+  arma_extra_debug_sigprint();
+
+  isnt_same_type<eT, typename T1::elem_type>::check();
+  
+  eop_type::apply_inplace_plus(*this, X);
+  
+  return *this;
+  }
+
+
+
+template<typename eT>
+template<typename T1, typename eop_type>
+inline
+const Mat<eT>&
+Mat<eT>::operator-=(const eOp<T1, eop_type>& X)
+  {
+  arma_extra_debug_sigprint();
+
+  isnt_same_type<eT, typename T1::elem_type>::check();
+  
+  eop_type::apply_inplace_minus(*this, X);
+  
+  return *this;
+  }
+
+
+
+template<typename eT>
+template<typename T1, typename eop_type>
+inline
+const Mat<eT>&
+Mat<eT>::operator*=(const eOp<T1, eop_type>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  isnt_same_type<eT, typename T1::elem_type>::check();
+  
+  glue_times::apply_inplace(*this, X);
+  
+  return *this;
+  }
+
+
+
+template<typename eT>
+template<typename T1, typename eop_type>
+inline
+const Mat<eT>&
+Mat<eT>::operator%=(const eOp<T1, eop_type>& X)
+  {
+  arma_extra_debug_sigprint();
+
+  isnt_same_type<eT, typename T1::elem_type>::check();
+  
+  eop_type::apply_inplace_schur(*this, X);
+  
+  return *this;
+  }
+
+
+
+template<typename eT>
+template<typename T1, typename eop_type>
+inline
+const Mat<eT>&
+Mat<eT>::operator/=(const eOp<T1, eop_type>& X)
+  {
+  arma_extra_debug_sigprint();
+
+  isnt_same_type<eT, typename T1::elem_type>::check();
+  
+  eop_type::apply_inplace_div(*this, X);
+  
+  return *this;
+  }
+
+
+
+//! EXPERIMENTAL
+template<typename eT>
+template<typename T1, typename op_type>
+inline
+Mat<eT>::Mat(const mtOp<eT, T1, op_type>& X)
+  : n_rows(0)
+  , n_cols(0)
+  , n_elem(0)
+  , use_aux_mem(false)
+  //, mem(0)
+  , mem(mem)
+  {
+  arma_extra_debug_sigprint_this(this);
+  
+  op_type::apply(*this, X);
+  }
+
+
+
+//! EXPERIMENTAL
+template<typename eT>
+template<typename T1, typename op_type>
+inline
+const Mat<eT>&
+Mat<eT>::operator=(const mtOp<eT, T1, op_type>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  op_type::apply(*this, X);
+  
+  return *this;
+  }
+
+
+
+//! EXPERIMENTAL
+template<typename eT>
+template<typename T1, typename op_type>
+inline
+const Mat<eT>&
+Mat<eT>::operator+=(const mtOp<eT, T1, op_type>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  const Mat<eT> m(X);
+  
+  return (*this).operator+=(m);
+  }
+
+
+
+//! EXPERIMENTAL
+template<typename eT>
+template<typename T1, typename op_type>
+inline
+const Mat<eT>&
+Mat<eT>::operator-=(const mtOp<eT, T1, op_type>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  const Mat<eT> m(X);
+  
+  return (*this).operator-=(m);
+  }
+
+
+
+//! EXPERIMENTAL
+template<typename eT>
+template<typename T1, typename op_type>
+inline
+const Mat<eT>&
+Mat<eT>::operator*=(const mtOp<eT, T1, op_type>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  const Mat<eT> m(X);
+  
+  return (*this).operator*=(m);
+  }
+
+
+
+//! EXPERIMENTAL
+template<typename eT>
+template<typename T1, typename op_type>
+inline
+const Mat<eT>&
+Mat<eT>::operator%=(const mtOp<eT, T1, op_type>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  const Mat<eT> m(X);
+  
+  return (*this).operator%=(m);
+  }
+
+
+
+//! EXPERIMENTAL
+template<typename eT>
+template<typename T1, typename op_type>
+inline
+const Mat<eT>&
+Mat<eT>::operator/=(const mtOp<eT, T1, op_type>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  const Mat<eT> m(X);
+  
+  return (*this).operator/=(m);
+  }
+
+
+
 //! create a matrix from Glue, i.e. run the previously delayed binary operations
 template<typename eT>
 template<typename T1, typename T2, typename glue_type>
@@ -1263,7 +1694,11 @@ Mat<eT>::Mat(const Glue<T1, T2, glue_type>& X)
   , mem(mem)
   {
   arma_extra_debug_sigprint_this(this);
-  this->operator=(X);
+  
+  isnt_same_type<eT, typename T1::elem_type>::check();
+  isnt_same_type<eT, typename T2::elem_type>::check();
+  
+  glue_type::apply(*this, X);
   }
 
 
@@ -1277,11 +1712,6 @@ Mat<eT>::operator=(const Glue<T1, T2, glue_type>& X)
   {
   arma_extra_debug_sigprint();
   
-  // TODO:
-  // it may be simpler to pass the two objects (currently wrapped in X)
-  // directly to the apply function.
-  // (many adjustments throughout the source code will be required)
-  
   isnt_same_type<eT, typename T1::elem_type>::check();
   isnt_same_type<eT, typename T2::elem_type>::check();
   
@@ -1291,6 +1721,7 @@ Mat<eT>::operator=(const Glue<T1, T2, glue_type>& X)
   }
 
 
+
 //! in-place matrix addition, with the right-hand-side operands having delayed operations
 template<typename eT>
 template<typename T1, typename T2, typename glue_type>
@@ -1303,9 +1734,9 @@ Mat<eT>::operator+=(const Glue<T1, T2, glue_type>& X)
   isnt_same_type<eT, typename T1::elem_type>::check();
   isnt_same_type<eT, typename T2::elem_type>::check();
   
-  glue_plus::apply_inplace(*this, X);
+  const Mat<eT> m(X);
   
-  return *this;
+  return (*this).operator+=(m);
   }
 
 
@@ -1322,9 +1753,9 @@ Mat<eT>::operator-=(const Glue<T1, T2, glue_type>& X)
   isnt_same_type<eT, typename T1::elem_type>::check();
   isnt_same_type<eT, typename T2::elem_type>::check();
   
-  glue_minus::apply_inplace(*this, X);
+  const Mat<eT> m(X);
   
-  return *this;
+  return (*this).operator-=(m);
   }
 
 
@@ -1342,6 +1773,7 @@ Mat<eT>::operator*=(const Glue<T1, T2, glue_type>& X)
   isnt_same_type<eT, typename T2::elem_type>::check();
   
   glue_times::apply_inplace(*this, X);
+  
   return *this;
   }
 
@@ -1359,8 +1791,9 @@ Mat<eT>::operator%=(const Glue<T1, T2, glue_type>& X)
   isnt_same_type<eT, typename T1::elem_type>::check();
   isnt_same_type<eT, typename T2::elem_type>::check();
   
-  glue_schur::apply_inplace(*this, X);
-  return *this;
+  const Mat<eT> m(X);
+  
+  return (*this).operator%=(m);
   }
 
 
@@ -1377,19 +1810,297 @@ Mat<eT>::operator/=(const Glue<T1, T2, glue_type>& X)
   isnt_same_type<eT, typename T1::elem_type>::check();
   isnt_same_type<eT, typename T2::elem_type>::check();
   
-  glue_div::apply_inplace(*this, X);
+  const Mat<eT> m(X);
+  
+  return (*this).operator/=(m);
+  }
+
+
+
+template<typename eT>
+template<typename T1, typename T2>
+inline
+const Mat<eT>&
+Mat<eT>::operator+=(const Glue<T1, T2, glue_times>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  glue_times::apply_inplace_plus(*this, X, s32(+1));
+  
   return *this;
   }
 
 
 
+template<typename eT>
+template<typename T1, typename T2>
+inline
+const Mat<eT>&
+Mat<eT>::operator-=(const Glue<T1, T2, glue_times>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  glue_times::apply_inplace_plus(*this, X, s32(-1));
+  
+  return *this;
+  }
+
+
+
+//! create a matrix from eGlue, i.e. run the previously delayed binary operations
+template<typename eT>
+template<typename T1, typename T2, typename eglue_type>
+inline
+Mat<eT>::Mat(const eGlue<T1, T2, eglue_type>& X)
+  : n_rows(0)
+  , n_cols(0)
+  , n_elem(0)
+  , use_aux_mem(false)
+  //, mem(0)
+  , mem(mem)
+  {
+  arma_extra_debug_sigprint_this(this);
+  
+  isnt_same_type<eT, typename T1::elem_type>::check();
+  isnt_same_type<eT, typename T2::elem_type>::check();
+  
+  eglue_type::apply(*this, X);
+  }
+
+
+
+//! create a matrix from eGlue, i.e. run the previously delayed binary operations
+template<typename eT>
+template<typename T1, typename T2, typename eglue_type>
+inline
+const Mat<eT>&
+Mat<eT>::operator=(const eGlue<T1, T2, eglue_type>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  isnt_same_type<eT, typename T1::elem_type>::check();
+  isnt_same_type<eT, typename T2::elem_type>::check();
+  
+  eglue_type::apply(*this, X);
+  
+  return *this;
+  }
+
+
+
+//! in-place matrix addition, with the right-hand-side operands having delayed operations
+template<typename eT>
+template<typename T1, typename T2, typename eglue_type>
+inline
+const Mat<eT>&
+Mat<eT>::operator+=(const eGlue<T1, T2, eglue_type>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  isnt_same_type<eT, typename T1::elem_type>::check();
+  isnt_same_type<eT, typename T2::elem_type>::check();
+  
+  eglue_type::apply_inplace_plus(*this, X);
+  
+  return *this;
+  }
+
+
+
+//! in-place matrix subtraction, with the right-hand-side operands having delayed operations
+template<typename eT>
+template<typename T1, typename T2, typename eglue_type>
+inline
+const Mat<eT>&
+Mat<eT>::operator-=(const eGlue<T1, T2, eglue_type>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  isnt_same_type<eT, typename T1::elem_type>::check();
+  isnt_same_type<eT, typename T2::elem_type>::check();
+  
+  eglue_type::apply_inplace_minus(*this, X);
+  
+  return *this;
+  }
+
+
+
+template<typename eT>
+template<typename T1, typename T2, typename eglue_type>
+inline
+const Mat<eT>&
+Mat<eT>::operator*=(const eGlue<T1, T2, eglue_type>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  isnt_same_type<eT, typename T1::elem_type>::check();
+  isnt_same_type<eT, typename T2::elem_type>::check();
+  
+  glue_times::apply_inplace(*this, X);
+  return *this;
+  }
+
+
+
+template<typename eT>
+template<typename T1, typename T2, typename eglue_type>
+inline
+const Mat<eT>&
+Mat<eT>::operator%=(const eGlue<T1, T2, eglue_type>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  isnt_same_type<eT, typename T1::elem_type>::check();
+  isnt_same_type<eT, typename T2::elem_type>::check();
+  
+  eglue_type::apply_inplace_schur(*this, X);
+  return *this;
+  }
+
+
+
+template<typename eT>
+template<typename T1, typename T2, typename eglue_type>
+inline
+const Mat<eT>&
+Mat<eT>::operator/=(const eGlue<T1, T2, eglue_type>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  isnt_same_type<eT, typename T1::elem_type>::check();
+  isnt_same_type<eT, typename T2::elem_type>::check();
+  
+  eglue_type::apply_inplace_div(*this, X);
+  return *this;
+  }
+
+
+
+//! EXPERIMENTAL: create a matrix from mtGlue, i.e. run the previously delayed binary operations
+template<typename eT>
+template<typename T1, typename T2, typename glue_type>
+inline
+Mat<eT>::Mat(const mtGlue<eT, T1, T2, glue_type>& X)
+  : n_rows(0)
+  , n_cols(0)
+  , n_elem(0)
+  , use_aux_mem(false)
+  //, mem(0)
+  , mem(mem)
+  {
+  arma_extra_debug_sigprint_this(this);
+  
+  glue_type::apply(*this, X);
+  }
+
+
+
+//! EXPERIMENTAL: create a matrix from Glue, i.e. run the previously delayed binary operations
+template<typename eT>
+template<typename T1, typename T2, typename glue_type>
+inline
+const Mat<eT>&
+Mat<eT>::operator=(const mtGlue<eT, T1, T2, glue_type>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  glue_type::apply(*this, X);
+  
+  return *this;
+  }
+
+
+
+//! EXPERIMENTAL: in-place matrix addition, with the right-hand-side operands having delayed operations
+template<typename eT>
+template<typename T1, typename T2, typename glue_type>
+inline
+const Mat<eT>&
+Mat<eT>::operator+=(const mtGlue<eT, T1, T2, glue_type>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  const Mat<eT> m(X);
+  
+  return (*this).operator+=(m);
+  }
+
+
+
+//! EXPERIMENTAL: in-place matrix subtraction, with the right-hand-side operands having delayed operations
+template<typename eT>
+template<typename T1, typename T2, typename glue_type>
+inline
+const Mat<eT>&
+Mat<eT>::operator-=(const mtGlue<eT, T1, T2, glue_type>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  const Mat<eT> m(X);
+  
+  return (*this).operator-=(m);
+  }
+
+
+
+//! EXPERIMENTAL: in-place matrix multiplications, with the right-hand-side operands having delayed operations
+template<typename eT>
+template<typename T1, typename T2, typename glue_type>
+inline
+const Mat<eT>&
+Mat<eT>::operator*=(const mtGlue<eT, T1, T2, glue_type>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  const Mat<eT> m(X);
+  
+  glue_times::apply_inplace(*this, m);
+  
+  return *this;
+  }
+
+
+
+//! EXPERIMENTAL: in-place matrix element-wise multiplication, with the right-hand-side operands having delayed operations
+template<typename eT>
+template<typename T1, typename T2, typename glue_type>
+inline
+const Mat<eT>&
+Mat<eT>::operator%=(const mtGlue<eT, T1, T2, glue_type>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  const Mat<eT> m(X);
+  
+  return (*this).operator%=(m);
+  }
+
+
+
+//! EXPERIMENTAL: in-place matrix element-wise division, with the right-hand-side operands having delayed operations
+template<typename eT>
+template<typename T1, typename T2, typename glue_type>
+inline
+const Mat<eT>&
+Mat<eT>::operator/=(const mtGlue<eT, T1, T2, glue_type>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  const Mat<eT> m(X);
+  
+  return (*this).operator/=(m);
+  }
+
+
+
 //! linear element accessor (treats the matrix as a vector); bounds checking not done when ARMA_NO_DEBUG is defined
 template<typename eT>
 arma_inline
 eT&
 Mat<eT>::operator() (const u32 i)
   {
-  arma_debug_check( (i >= n_elem), "Mat::operator(): index out of bounds");
+  arma_debug_check( (i >= n_elem), "Mat::operator(): out of bounds");
   return access::rw(mem[i]);
   }
 
@@ -1401,7 +2112,7 @@ arma_inline
 eT
 Mat<eT>::operator() (const u32 i) const
   {
-  arma_debug_check( (i >= n_elem), "Mat::operator(): index out of bounds");
+  arma_debug_check( (i >= n_elem), "Mat::operator(): out of bounds");
   return mem[i];
   }
 
@@ -1434,7 +2145,7 @@ arma_inline
 eT&
 Mat<eT>::operator() (const u32 in_row, const u32 in_col)
   {
-  arma_debug_check( ((in_row >= n_rows) || (in_col >= n_cols)), "Mat::operator(): index out of bounds");
+  arma_debug_check( ((in_row >= n_rows) || (in_col >= n_cols)), "Mat::operator(): out of bounds");
   return access::rw(mem[in_row + in_col*n_rows]);
   }
 
@@ -1446,7 +2157,7 @@ arma_inline
 eT
 Mat<eT>::operator() (const u32 in_row, const u32 in_col) const
   {
-  arma_debug_check( ((in_row >= n_rows) || (in_col >= n_cols)), "Mat::operator(): index out of bounds");
+  arma_debug_check( ((in_row >= n_rows) || (in_col >= n_cols)), "Mat::operator(): out of bounds");
   return mem[in_row + in_col*n_rows];
   }
 
@@ -1810,15 +2521,27 @@ Mat<eT>::copy_size(const Mat<eT2>& m)
 
 //! fill the matrix with the specified value
 template<typename eT>
+arma_hot
 inline
 void
 Mat<eT>::fill(const eT val)
   {
   arma_extra_debug_sigprint();
   
-  for(u32 i=0; i<n_elem; ++i)
+        eT* local_ptr    = memptr();
+  const u32 local_n_elem = n_elem;
+  
+  u32 i,j;
+  
+  for(i=0, j=1; j<local_n_elem; i+=2, j+=2)
     {
-    access::rw(mem[i]) = val;
+    local_ptr[i] = val;
+    local_ptr[j] = val;
+    }
+  
+  if(i < local_n_elem)
+    {
+    local_ptr[i] = val;
     }
   }
 
@@ -1889,33 +2612,79 @@ Mat<eT>::reset()
 //! save the matrix to a file
 template<typename eT>
 inline
-void
-Mat<eT>::save(const std::string name, const file_type type) const
+bool
+Mat<eT>::save(const std::string name, const file_type type, const bool print_status) const
   {
   arma_extra_debug_sigprint();
   
+  bool save_okay;
+  
   switch(type)
     {
     case raw_ascii:
-      diskio::save_raw_ascii(*this, name);
+      save_okay = diskio::save_raw_ascii(*this, name);
       break;
     
     case arma_ascii:
-      diskio::save_arma_ascii(*this, name);
+      save_okay = diskio::save_arma_ascii(*this, name);
       break;
     
     case arma_binary:
-      diskio::save_arma_binary(*this, name);
+      save_okay = diskio::save_arma_binary(*this, name);
       break;
       
     case pgm_binary:
-      diskio::save_pgm_binary(*this, name);
+      save_okay = diskio::save_pgm_binary(*this, name);
       break;
     
     default:
-      arma_stop("Mat::save(): unsupported file type");
+      arma_warn(print_status, "Mat::save(): unsupported file type");
+      save_okay = false;
     }
   
+  arma_warn( (print_status && (save_okay == false)), "Mat::save(): couldn't write to ", name);
+  
+  return save_okay;
+  }
+
+
+
+//! save the matrix to a stream
+template<typename eT>
+inline
+bool
+Mat<eT>::save(std::ostream& os, const file_type type, const bool print_status) const
+  {
+  arma_extra_debug_sigprint();
+  
+  bool save_okay;
+  
+  switch(type)
+    {
+    case raw_ascii:
+      save_okay = diskio::save_raw_ascii(*this, os);
+      break;
+    
+    case arma_ascii:
+      save_okay = diskio::save_arma_ascii(*this, os);
+      break;
+    
+    case arma_binary:
+      save_okay = diskio::save_arma_binary(*this, os);
+      break;
+      
+    case pgm_binary:
+      save_okay = diskio::save_pgm_binary(*this, os);
+      break;
+    
+    default:
+      arma_warn(print_status, "Mat::save(): unsupported file type");
+      save_okay = false;
+    }
+  
+  arma_warn( (print_status && (save_okay == false)), "Mat::save(): couldn't write to the given stream");
+  
+  return save_okay;
   }
 
 
@@ -1923,37 +2692,550 @@ Mat<eT>::save(const std::string name, const file_type type) const
 //! load a matrix from a file
 template<typename eT>
 inline
-void
-Mat<eT>::load(const std::string name, const file_type type)
+bool
+Mat<eT>::load(const std::string name, const file_type type, const bool print_status)
   {
   arma_extra_debug_sigprint();
   
+  bool load_okay;
+  std::string err_msg;
+  
   switch(type)
     {
     case auto_detect:
-      diskio::load_auto_detect(*this, name);
+      load_okay = diskio::load_auto_detect(*this, name, err_msg);
       break;
     
     case raw_ascii:
-      diskio::load_raw_ascii(*this, name);
+      load_okay = diskio::load_raw_ascii(*this, name, err_msg);
       break;
     
     case arma_ascii:
-      diskio::load_arma_ascii(*this, name);
+      load_okay = diskio::load_arma_ascii(*this, name, err_msg);
       break;
     
     case arma_binary:
-      diskio::load_arma_binary(*this, name);
+      load_okay = diskio::load_arma_binary(*this, name, err_msg);
       break;
       
     case pgm_binary:
-      diskio::load_pgm_binary(*this, name);
+      load_okay = diskio::load_pgm_binary(*this, name, err_msg);
       break;
     
     default:
-      arma_stop("Mat::load(): unsupported file type");
+      arma_warn(print_status, "Mat::load(): unsupported file type");
+      load_okay = false;
     }
   
+  if( (print_status == true) && (load_okay == false) )
+    {
+    if(err_msg.length() > 0)
+      {
+      arma_print("Mat::load(): ", err_msg, name);
+      }
+    else
+      {
+      arma_print("Mat::load(): couldn't read ", name);
+      }
+    }
+  
+  if(load_okay == false)
+    {
+    (*this).reset();
+    }
+    
+  return load_okay;
+  }
+
+
+
+//! load a matrix from a stream
+template<typename eT>
+inline
+bool
+Mat<eT>::load(std::istream& is, const file_type type, const bool print_status)
+  {
+  arma_extra_debug_sigprint();
+  
+  bool load_okay;
+  std::string err_msg;
+  
+  switch(type)
+    {
+    case auto_detect:
+      load_okay = diskio::load_auto_detect(*this, is, err_msg);
+      break;
+    
+    case raw_ascii:
+      load_okay = diskio::load_raw_ascii(*this, is, err_msg);
+      break;
+    
+    case arma_ascii:
+      load_okay = diskio::load_arma_ascii(*this, is, err_msg);
+      break;
+    
+    case arma_binary:
+      load_okay = diskio::load_arma_binary(*this, is, err_msg);
+      break;
+      
+    case pgm_binary:
+      load_okay = diskio::load_pgm_binary(*this, is, err_msg);
+      break;
+    
+    default:
+      arma_warn(print_status, "Mat::load(): unsupported file type");
+      load_okay = false;
+    }
+  
+  
+  if( (print_status == true) && (load_okay == false) )
+    {
+    if(err_msg.length() > 0)
+      {
+      arma_print("Mat::load(): ", err_msg, "the given stream");
+      }
+    else
+      {
+      arma_print("Mat::load(): couldn't load from the given stream");
+      }
+    }
+  
+  if(load_okay == false)
+    {
+    (*this).reset();
+    }
+    
+  return load_okay;
+  }
+
+
+
+//! save the matrix to a file, without printing any error messages
+template<typename eT>
+inline
+bool
+Mat<eT>::quiet_save(const std::string name, const file_type type) const
+  {
+  arma_extra_debug_sigprint();
+  
+  return (*this).save(name, type, false);
+  }
+
+
+
+//! save the matrix to a stream, without printing any error messages
+template<typename eT>
+inline
+bool
+Mat<eT>::quiet_save(std::ostream& os, const file_type type) const
+  {
+  arma_extra_debug_sigprint();
+  
+  return (*this).save(os, type, false);
+  }
+
+
+
+//! load a matrix from a file, without printing any error messages
+template<typename eT>
+inline
+bool
+Mat<eT>::quiet_load(const std::string name, const file_type type)
+  {
+  arma_extra_debug_sigprint();
+  
+  return (*this).load(name, type, false);
+  }
+
+
+
+//! load a matrix from a stream, without printing any error messages
+template<typename eT>
+inline
+bool
+Mat<eT>::quiet_load(std::istream& is, const file_type type)
+  {
+  arma_extra_debug_sigprint();
+  
+  return (*this).load(is, type, false);
+  }
+
+
+
+template<typename eT>
+inline
+Mat<eT>::row_iterator::row_iterator(Mat<eT>& in_M, const u32 in_row)
+  : M  (in_M  )
+  , row(in_row)
+  , col(0     )
+  {
+  arma_extra_debug_sigprint();
+  }
+
+
+
+template<typename eT>
+inline
+eT&
+Mat<eT>::row_iterator::operator*()
+  {
+  return M.at(row,col);
+  }
+
+
+
+template<typename eT>
+inline
+typename Mat<eT>::row_iterator&
+Mat<eT>::row_iterator::operator++()
+  {
+  ++col;
+  
+  if(col >= M.n_cols)
+    {
+    col = 0;
+    ++row;
+    }
+  
+  return *this;
+  }
+
+
+
+template<typename eT>
+inline
+void
+Mat<eT>::row_iterator::operator++(int)
+  {
+  operator++();
+  }
+
+
+
+template<typename eT>
+inline
+typename Mat<eT>::row_iterator&
+Mat<eT>::row_iterator::operator--()
+  {
+  if(col > 0)
+    {
+    --col;
+    }
+  else
+    {
+    if(row > 0)
+      {
+      col = M.n_cols - 1;
+      --row;
+      }
+    }
+  
+  return *this;
+  }
+
+
+
+template<typename eT>
+inline
+void
+Mat<eT>::row_iterator::operator--(int)
+  {
+  operator--();
+  }
+
+
+
+template<typename eT>
+inline
+bool
+Mat<eT>::row_iterator::operator!=(const typename Mat<eT>::row_iterator& X) const
+  {
+  return ( (row != X.row) || (col != X.col) ) ? true : false;
+  }
+
+
+
+template<typename eT>
+inline
+bool
+Mat<eT>::row_iterator::operator==(const typename Mat<eT>::row_iterator& X) const
+  {
+  return ( (row == X.row) && (col == X.col) ) ? true : false;
+  }
+
+
+
+template<typename eT>
+inline
+Mat<eT>::const_row_iterator::const_row_iterator(const Mat<eT>& in_M, const u32 in_row)
+  : M  (in_M  )
+  , row(in_row)
+  , col(0     )
+  {
+  arma_extra_debug_sigprint();
+  }
+
+
+
+template<typename eT>
+inline
+Mat<eT>::const_row_iterator::const_row_iterator(const typename Mat<eT>::row_iterator& X)
+  : M  (X.M)
+  , row(X.row)
+  , col(X.col)
+  {
+  arma_extra_debug_sigprint();
+  }
+
+
+
+template<typename eT>
+inline
+eT
+Mat<eT>::const_row_iterator::operator*() const
+  {
+  return M.at(row,col);
+  }
+
+
+
+template<typename eT>
+inline
+typename Mat<eT>::const_row_iterator&
+Mat<eT>::const_row_iterator::operator++()
+  {
+  ++col;
+  
+  if(col >= M.n_cols)
+    {
+    col = 0;
+    ++row;
+    }
+  
+  return *this;
+  }
+
+
+
+template<typename eT>
+inline
+void
+Mat<eT>::const_row_iterator::operator++(int)
+  {
+  operator++();
+  }
+
+
+
+template<typename eT>
+inline
+typename Mat<eT>::const_row_iterator&
+Mat<eT>::const_row_iterator::operator--()
+  {
+  if(col > 0)
+    {
+    --col;
+    }
+  else
+    {
+    if(row > 0)
+      {
+      col = M.n_cols - 1;
+      --row;
+      }
+    }
+  
+  return *this;
+  }
+
+
+
+template<typename eT>
+inline
+void
+Mat<eT>::const_row_iterator::operator--(int)
+  {
+  operator--();
+  }
+
+
+
+template<typename eT>
+inline
+bool
+Mat<eT>::const_row_iterator::operator!=(const typename Mat<eT>::const_row_iterator& X) const
+  {
+  return ( (row != X.row) || (col != X.col) ) ? true : false;
+  }
+
+
+
+template<typename eT>
+inline
+bool
+Mat<eT>::const_row_iterator::operator==(const typename Mat<eT>::const_row_iterator& X) const
+  {
+  return ( (row == X.row) && (col == X.col) ) ? true : false;
+  }
+
+
+
+template<typename eT>
+inline
+typename Mat<eT>::iterator
+Mat<eT>::begin()
+  {
+  arma_extra_debug_sigprint();
+  
+  return memptr();
+  }
+
+
+
+template<typename eT>
+inline
+typename Mat<eT>::const_iterator
+Mat<eT>::begin() const
+  {
+  arma_extra_debug_sigprint();
+  
+  return memptr();
+  }
+
+
+
+template<typename eT>
+inline
+typename Mat<eT>::iterator
+Mat<eT>::end()
+  {
+  arma_extra_debug_sigprint();
+  
+  return memptr() + n_elem;
+  }
+
+
+
+template<typename eT>
+inline
+typename Mat<eT>::const_iterator
+Mat<eT>::end() const
+  {
+  arma_extra_debug_sigprint();
+  
+  return memptr() + n_elem;
+  }
+  
+
+
+template<typename eT>
+inline
+typename Mat<eT>::col_iterator
+Mat<eT>::begin_col(const u32 col_num)
+  {
+  arma_extra_debug_sigprint();
+  
+  arma_debug_check( (col_num >= n_cols), "begin_col(): index out of bounds");
+  
+  return colptr(col_num);
+  }
+
+
+
+template<typename eT>
+inline
+typename Mat<eT>::const_col_iterator
+Mat<eT>::begin_col(const u32 col_num) const
+  {
+  arma_extra_debug_sigprint();
+  
+  arma_debug_check( (col_num >= n_cols), "begin_col(): index out of bounds");
+  
+  return colptr(col_num);
+  }
+
+
+
+template<typename eT>
+inline
+typename Mat<eT>::col_iterator
+Mat<eT>::end_col(const u32 col_num)
+  {
+  arma_extra_debug_sigprint();
+  
+  arma_debug_check( (col_num >= n_cols), "end_col(): index out of bounds");
+  
+  return colptr(col_num) + n_rows;
+  }
+
+
+
+template<typename eT>
+inline
+typename Mat<eT>::const_col_iterator
+Mat<eT>::end_col(const u32 col_num) const
+  {
+  arma_extra_debug_sigprint();
+  
+  arma_debug_check( (col_num >= n_cols), "end_col(): index out of bounds");
+  
+  return colptr(col_num) + n_rows;
+  }
+  
+
+
+template<typename eT>
+inline
+typename Mat<eT>::row_iterator
+Mat<eT>::begin_row(const u32 row_num)
+  {
+  arma_extra_debug_sigprint();
+  
+  arma_debug_check( (row_num >= n_rows), "Mat::begin_row(): index out of bounds" );
+  
+  return typename Mat<eT>::row_iterator(*this, row_num);
+  }
+
+
+
+template<typename eT>
+inline
+typename Mat<eT>::const_row_iterator
+Mat<eT>::begin_row(const u32 row_num) const
+  {
+  arma_extra_debug_sigprint();
+  
+  arma_debug_check( (row_num >= n_rows), "Mat::begin_row(): index out of bounds" );
+  
+  return typename Mat<eT>::const_row_iterator(*this, row_num);
+  }
+
+
+
+template<typename eT>
+inline
+typename Mat<eT>::row_iterator
+Mat<eT>::end_row(const u32 row_num)
+  {
+  arma_extra_debug_sigprint();
+  
+  arma_debug_check( (row_num >= n_rows), "Mat::end_row(): index out of bounds" );
+  
+  return typename Mat<eT>::row_iterator(*this, row_num + 1);
+  }
+
+
+
+template<typename eT>
+inline
+typename Mat<eT>::const_row_iterator
+Mat<eT>::end_row(const u32 row_num) const
+  {
+  arma_extra_debug_sigprint();
+  
+  arma_debug_check( (row_num >= n_rows), "Mat::end_row(): index out of bounds" );
+  
+  return typename Mat<eT>::const_row_iterator(*this, row_num + 1);
   }
 
 
@@ -1966,8 +3248,16 @@ Mat_aux::prefix_pp(Mat<eT>& x)
   {
         eT* memptr = x.memptr();
   const u32 n_elem = x.n_elem;
+  
+  u32 i,j;
 
-  for(u32 i=0; i<n_elem; ++i)
+  for(i=0, j=1; j<n_elem; i+=2, j+=2)
+    {
+    ++(memptr[i]);
+    ++(memptr[j]);
+    }
+  
+  if(i < n_elem)
     {
     ++(memptr[i]);
     }
@@ -1994,8 +3284,16 @@ Mat_aux::postfix_pp(Mat<eT>& x)
   {
         eT* memptr = x.memptr();
   const u32 n_elem = x.n_elem;
-
-  for(u32 i=0; i<n_elem; ++i)
+  
+  u32 i,j;
+  
+  for(i=0, j=1; j<n_elem; i+=2, j+=2)
+    {
+    (memptr[i])++;
+    (memptr[j])++;
+    }
+  
+  if(i < n_elem)
     {
     (memptr[i])++;
     }
@@ -2023,7 +3321,15 @@ Mat_aux::prefix_mm(Mat<eT>& x)
         eT* memptr = x.memptr();
   const u32 n_elem = x.n_elem;
 
-  for(u32 i=0; i<n_elem; ++i)
+  u32 i,j;
+
+  for(i=0, j=1; j<n_elem; i+=2, j+=2)
+    {
+    --(memptr[i]);
+    --(memptr[j]);
+    }
+  
+  if(i < n_elem)
     {
     --(memptr[i]);
     }
@@ -2051,7 +3357,15 @@ Mat_aux::postfix_mm(Mat<eT>& x)
         eT* memptr = x.memptr();
   const u32 n_elem = x.n_elem;
 
-  for(u32 i=0; i<n_elem; ++i)
+  u32 i,j;
+
+  for(i=0, j=1; j<n_elem; i+=2, j+=2)
+    {
+    (memptr[i])--;
+    (memptr[j])--;
+    }
+  
+  if(i < n_elem)
     {
     (memptr[i])--;
     }
