@@ -15,6 +15,23 @@
 
 
 
+#if !defined(ARMA_USE_LAPACK)
+// #define ARMA_USE_LAPACK
+#endif
+
+#if !defined(ARMA_USE_BLAS)
+// #define ARMA_USE_BLAS
+#endif
+
+// #define ARMA_USE_ATLAS
+// #define ARMA_ATLAS_INCLUDE_DIR /usr/include/
+//// If you're using ATLAS and the compiler can't find cblas.h and/or clapack.h
+//// uncomment the above define and specify the appropriate include directory.
+//// Make sure the directory has a trailing /
+
+// #define ARMA_USE_BOOST
+// #define ARMA_USE_BOOST_DATE
+
 // #define ARMA_HAVE_STD_ISFINITE
 // #define ARMA_HAVE_STD_ISINF
 // #define ARMA_HAVE_STD_ISNAN
@@ -23,17 +40,5 @@
 // #define ARMA_HAVE_LOG1P
 // #define ARMA_HAVE_GETTIMEOFDAY
 
-// #define ARMA_USE_ATLAS
-// #define ARMA_USE_LAPACK
-// #define ARMA_USE_BLAS
-// #define ARMA_USE_BOOST
-// #define ARMA_USE_BOOST_DATE
-
 // #define ARMA_EXTRA_DEBUG
 // #define ARMA_NO_DEBUG
-
-#if defined(ARMA_USE_ATLAS)
-  #if !defined(ARMA_ATLAS_INCLUDE_DIR)
-    #define ARMA_ATLAS_INCLUDE_DIR
-  #endif
-#endif
