@@ -1,7 +1,9 @@
-// Copyright (C) 2009 NICTA
+// Copyright (C) 2010 NICTA and the authors listed below
+// http://nicta.com.au
 // 
 // Authors:
 // - Conrad Sanderson (conradsand at ieee dot org)
+// - Ian Cullinan (ian dot cullinan at nicta dot com dot au)
 // 
 // This file is part of the Armadillo C++ library.
 // It is provided without any warranty of fitness
@@ -177,16 +179,14 @@ field<oT>::set_size(const u32 n_rows_in, const u32 n_cols_in)
 
 //! change the field to have the specified dimensions (data is not preserved)
 template<typename oT>
+template<typename oT2>
 inline
 void
-field<oT>::copy_size(const field<oT>& x)
+field<oT>::copy_size(const field<oT2>& x)
   {
   arma_extra_debug_sigprint();
   
-  if(this != &x)
-    {
-    init(x.n_rows, x.n_cols);
-    }
+  init(x.n_rows, x.n_cols);
   }
 
 
@@ -556,24 +556,166 @@ field<oT>::reset_objects()
 
 template<typename oT>
 inline
-void
-field<oT>::save(const std::string name, const file_type type) const
+bool
+field<oT>::save(const std::string name, const file_type type, const bool print_status) const
   {
   arma_extra_debug_sigprint();
   
-  field_aux::save(*this, name, type);
+  std::string err_msg;
+  const bool save_okay = field_aux::save(*this, name, type, err_msg);
+  
+  if( (print_status == true) && (save_okay == false) )
+    {
+    if(err_msg.length() > 0)
+      {
+      arma_print("field::save(): ", err_msg, name);
+      }
+    else
+      {
+      arma_print("field::save(): couldn't write to ", name);
+      }
+    }
+  
+  return save_okay;
   }
 
 
 
 template<typename oT>
 inline
-void
-field<oT>::load(const std::string name, const file_type type)
+bool
+field<oT>::save(std::ostream& os, const file_type type, const bool print_status) const
   {
   arma_extra_debug_sigprint();
   
-  field_aux::load(*this, name, type);
+  std::string err_msg;
+  const bool save_okay = field_aux::save(*this, os, type, err_msg);
+  
+  if( (print_status == true) && (save_okay == false) )
+    {
+    if(err_msg.length() > 0)
+      {
+      arma_print("field::save(): ", err_msg, "[ostream]");
+      }
+    else
+      {
+      arma_print("field::save(): couldn't write to [ostream]");
+      }
+    }
+  
+  return save_okay;
+  }
+
+
+
+template<typename oT>
+inline
+bool
+field<oT>::load(const std::string name, const file_type type, const bool print_status)
+  {
+  arma_extra_debug_sigprint();
+  
+  std::string err_msg;
+  const bool load_okay = field_aux::load(*this, name, type, err_msg);
+  
+  if( (print_status == true) && (load_okay == false) )
+    {
+    if(err_msg.length() > 0)
+      {
+      arma_print("field::load(): ", err_msg, name);
+      }
+    else
+      {
+      arma_print("field::load(): couldn't read from ", name);
+      }
+    }
+  
+  if(load_okay == false)
+    {
+    (*this).reset();
+    }
+  
+  return load_okay;
+  }
+
+
+
+template<typename oT>
+inline
+bool
+field<oT>::load(std::istream& is, const file_type type, const bool print_status)
+  {
+  arma_extra_debug_sigprint();
+  
+  std::string err_msg;
+  const bool load_okay = field_aux::load(*this, is, type, err_msg);
+  
+  if( (print_status == true) && (load_okay == false) )
+    {
+    if(err_msg.length() > 0)
+      {
+      arma_print("field::load(): ", err_msg, "[istream]");
+      }
+    else
+      {
+      arma_print("field::load(): couldn't read from [istream]");
+      }
+    }
+  
+  if(load_okay == false)
+    {
+    (*this).reset();
+    }
+  
+  return load_okay;
+  }
+
+
+
+template<typename oT>
+inline
+bool
+field<oT>::quiet_save(const std::string name, const file_type type) const
+  {
+  arma_extra_debug_sigprint();
+  
+  return (*this).save(name, type, false);
+  }
+
+
+
+template<typename oT>
+inline
+bool
+field<oT>::quiet_save(std::ostream& os, const file_type type) const
+  {
+  arma_extra_debug_sigprint();
+  
+  return (*this).save(os, type, false);
+  }
+
+
+
+template<typename oT>
+inline
+bool
+field<oT>::quiet_load(const std::string name, const file_type type)
+  {
+  arma_extra_debug_sigprint();
+  
+  return (*this).load(name, type, false);
+  }
+
+
+
+template<typename oT>
+inline
+bool
+field<oT>::quiet_load(std::istream& is, const file_type type)
+  {
+  arma_extra_debug_sigprint();
+  
+  return (*this).load(is, type, false);
   }
 
 
@@ -696,6 +838,241 @@ field<oT>::create_objects()
 
 
 
+template<typename oT>
+inline
+field<oT>::iterator::iterator(field<oT>& in_M, const bool at_end)
+  : M(in_M)
+  , i( (at_end == false) ? 0 : in_M.n_elem )
+  {
+  arma_extra_debug_sigprint();
+  }
+
+
+
+template<typename oT>
+inline
+oT&
+field<oT>::iterator::operator*()
+  {
+  return M[i];
+  }
+
+
+
+template<typename oT>
+inline
+typename field<oT>::iterator&
+field<oT>::iterator::operator++()
+  {
+  ++i;
+  
+  return *this;
+  }
+
+
+
+template<typename oT>
+inline
+void
+field<oT>::iterator::operator++(int)
+  {
+  operator++();
+  }
+
+
+
+template<typename oT>
+inline
+typename field<oT>::iterator&
+field<oT>::iterator::operator--()
+  {
+  if(i > 0)
+    {
+    --i;
+    }
+  
+  return *this;
+  }
+
+
+
+template<typename oT>
+inline
+void
+field<oT>::iterator::operator--(int)
+  {
+  operator--();
+  }
+
+
+
+template<typename oT>
+inline
+bool
+field<oT>::iterator::operator!=(const typename field<oT>::iterator& X) const
+  {
+  return (i != X.i);
+  }
+
+
+
+template<typename oT>
+inline
+bool
+field<oT>::iterator::operator==(const typename field<oT>::iterator& X) const
+  {
+  return (i == X.i);
+  }
+
+
+
+template<typename oT>
+inline
+field<oT>::const_iterator::const_iterator(const field<oT>& in_M, const bool at_end)
+  : M(in_M)
+  , i( (at_end == false) ? 0 : in_M.n_elem )
+  {
+  arma_extra_debug_sigprint();
+  }
+
+
+
+template<typename oT>
+inline
+field<oT>::const_iterator::const_iterator(const typename field<oT>::iterator& X)
+  : M(X.M)
+  , i(X.i)
+  {
+  arma_extra_debug_sigprint();
+  }
+
+
+
+template<typename oT>
+inline
+const oT&
+field<oT>::const_iterator::operator*() const
+  {
+  return M[i];
+  }
+
+
+
+template<typename oT>
+inline
+typename field<oT>::const_iterator&
+field<oT>::const_iterator::operator++()
+  {
+  ++i;
+  
+  return *this;
+  }
+
+
+
+template<typename oT>
+inline
+void
+field<oT>::const_iterator::operator++(int)
+  {
+  operator++();
+  }
+
+
+
+template<typename oT>
+inline
+typename field<oT>::const_iterator&
+field<oT>::const_iterator::operator--()
+  {
+  if(i > 0)
+    {
+    --i;
+    }
+  
+  return *this;
+  }
+
+
+
+template<typename oT>
+inline
+void
+field<oT>::const_iterator::operator--(int)
+  {
+  operator--();
+  }
+
+
+
+template<typename oT>
+inline
+bool
+field<oT>::const_iterator::operator!=(const typename field<oT>::const_iterator& X) const
+  {
+  return (i != X.i);
+  }
+
+
+
+template<typename oT>
+inline
+bool
+field<oT>::const_iterator::operator==(const typename field<oT>::const_iterator& X) const
+  {
+  return (i == X.i);
+  }
+
+
+
+template<typename oT>
+inline
+typename field<oT>::iterator
+field<oT>::begin()
+  {
+  arma_extra_debug_sigprint();
+  
+  return field<oT>::iterator(*this);
+  }
+
+
+
+template<typename oT>
+inline
+typename field<oT>::const_iterator
+field<oT>::begin() const
+  {
+  arma_extra_debug_sigprint();
+  
+  return field<oT>::const_iterator(*this);
+  }
+
+
+
+template<typename oT>
+inline
+typename field<oT>::iterator
+field<oT>::end()
+  {
+  arma_extra_debug_sigprint();
+  
+  return field<oT>::iterator(*this, true);
+  }
+
+
+
+template<typename oT>
+inline
+typename field<oT>::const_iterator
+field<oT>::end() const
+  {
+  arma_extra_debug_sigprint();
+  
+  return field<oT>::const_iterator(*this, true);
+  }
+  
+
+
 //
 //
 //
@@ -797,48 +1174,80 @@ field_aux::reset_objects(field< std::string >& x)
 
 template<typename oT>
 inline
-void
-field_aux::save(const field<oT>& x, const std::string& name, const file_type type)
+bool
+field_aux::save(const field<oT>& x, const std::string& name, const file_type type, std::string& err_msg)
   {
   arma_extra_debug_sigprint();
   
-  arma_print("field_aux::save(): sorry, saving this type of field is currently not supported");
+  err_msg = " [sorry, saving/loading this type of field is currently not supported] filename = ";
+  
+  return false;
   }
 
 
 
 template<typename oT>
 inline
-void
-field_aux::load(field<oT>& x, const std::string& name, const file_type type)
+bool
+field_aux::save(const field<oT>& x, std::ostream& os, const file_type type, std::string& err_msg)
   {
   arma_extra_debug_sigprint();
   
-  arma_print("field_aux::load(): sorry, loading this type of field is currently not supported");
-  x.reset();
+  err_msg = " [sorry, saving/loading this type of field is currently not supported] filename = ";
+  
+  return false;
+  }
+
+
+
+template<typename oT>
+inline
+bool
+field_aux::load(field<oT>& x, const std::string& name, const file_type type, std::string& err_msg)
+  {
+  arma_extra_debug_sigprint();
+  
+  err_msg = " [sorry, saving/loading this type of field is currently not supported] filename = ";
+  
+  return false;
+  }
+
+
+
+template<typename oT>
+inline
+bool
+field_aux::load(field<oT>& x, std::istream& is, const file_type type, std::string& err_msg)
+  {
+  arma_extra_debug_sigprint();
+  
+  err_msg = " [sorry, saving/loading this type of field is currently not supported] filename = ";
+  
+  return false;
   }
 
 
 
 template<typename eT>
 inline
-void
-field_aux::save(const field< Mat<eT> >& x, const std::string& name, const file_type type)
+bool
+field_aux::save(const field< Mat<eT> >& x, const std::string& name, const file_type type, std::string& err_msg)
   {
   arma_extra_debug_sigprint();
   
   switch(type)
     {
     case arma_binary:
-      diskio::save_arma_binary(x, name);
+      return diskio::save_arma_binary(x, name);
       break;
       
     case ppm_binary:
-      diskio::save_ppm_binary(x, name);
+      return diskio::save_ppm_binary(x, name);
       break;
     
     default:
-      arma_stop("field_aux::save(): unsupported type");
+      err_msg = " [unsupported type] filename = ";
+      return false;
     }
   }
 
@@ -846,27 +1255,53 @@ field_aux::save(const field< Mat<eT> >& x, const std::string& name, const file_t
 
 template<typename eT>
 inline
-void
-field_aux::load(field< Mat<eT> >& x, const std::string& name, const file_type type)
+bool
+field_aux::save(const field< Mat<eT> >& x, std::ostream& os, const file_type type, std::string& err_msg)
+  {
+  arma_extra_debug_sigprint();
+  
+  switch(type)
+    {
+    case arma_binary:
+      return diskio::save_arma_binary(x, os);
+      break;
+      
+    case ppm_binary:
+      return diskio::save_ppm_binary(x, os);
+      break;
+    
+    default:
+      err_msg = " [unsupported type] filename = ";
+      return false;
+    }
+  }
+
+
+
+template<typename eT>
+inline
+bool
+field_aux::load(field< Mat<eT> >& x, const std::string& name, const file_type type, std::string& err_msg)
   {
   arma_extra_debug_sigprint();
   
   switch(type)
     {
     case auto_detect:
-      diskio::load_auto_detect(x, name);
+      return diskio::load_auto_detect(x, name, err_msg);
       break;
     
     case arma_binary:
-      diskio::load_arma_binary(x, name);
+      return diskio::load_arma_binary(x, name, err_msg);
       break;
       
     case ppm_binary:
-      diskio::load_ppm_binary(x, name);
+      return diskio::load_ppm_binary(x, name, err_msg);
       break;
     
     default:
-      arma_stop("field_aux::load(): unsupported type");
+      err_msg = " [unsupported type] filename = ";
+      return false;
     }
   }
 
@@ -874,51 +1309,28 @@ field_aux::load(field< Mat<eT> >& x, const std::string& name, const file_type ty
 
 template<typename eT>
 inline
-void
-field_aux::save(const field< Col<eT> >& x, const std::string& name, const file_type type)
-  {
-  arma_extra_debug_sigprint();
-  
-  switch(type)
-    {
-    case arma_binary:
-      diskio::save_arma_binary(x, name);
-      break;
-      
-    case ppm_binary:
-      diskio::save_ppm_binary(x, name);
-      break;
-    
-    default:
-      arma_stop("field_aux::save(): unsupported type");
-    }
-  }
-
-
-
-template<typename eT>
-inline
-void
-field_aux::load(field< Col<eT> >& x, const std::string& name, const file_type type)
+bool
+field_aux::load(field< Mat<eT> >& x, std::istream& is, const file_type type, std::string& err_msg)
   {
   arma_extra_debug_sigprint();
   
   switch(type)
     {
     case auto_detect:
-      diskio::load_auto_detect(x, name);
+      return diskio::load_auto_detect(x, is, err_msg);
       break;
     
     case arma_binary:
-      diskio::load_arma_binary(x, name);
+      return diskio::load_arma_binary(x, is, err_msg);
       break;
       
     case ppm_binary:
-      diskio::load_ppm_binary(x, name);
+      return diskio::load_ppm_binary(x, is, err_msg);
       break;
     
     default:
-      arma_stop("field_aux::load(): unsupported type");
+      err_msg = " [unsupported type] filename = ";
+      return false;
     }
   }
 
@@ -926,23 +1338,24 @@ field_aux::load(field< Col<eT> >& x, const std::string& name, const file_type ty
 
 template<typename eT>
 inline
-void
-field_aux::save(const field< Row<eT> >& x, const std::string& name, const file_type type)
+bool
+field_aux::save(const field< Col<eT> >& x, const std::string& name, const file_type type, std::string& err_msg)
   {
   arma_extra_debug_sigprint();
   
   switch(type)
     {
     case arma_binary:
-      diskio::save_arma_binary(x, name);
+      return diskio::save_arma_binary(x, name);
       break;
       
     case ppm_binary:
-      diskio::save_ppm_binary(x, name);
+      return diskio::save_ppm_binary(x, name);
       break;
     
     default:
-      arma_stop("field_aux::save(): unsupported type");
+      err_msg = " [unsupported type] filename = ";
+      return false;
     }
   }
 
@@ -950,27 +1363,53 @@ field_aux::save(const field< Row<eT> >& x, const std::string& name, const file_t
 
 template<typename eT>
 inline
-void
-field_aux::load(field< Row<eT> >& x, const std::string& name, const file_type type)
+bool
+field_aux::save(const field< Col<eT> >& x, std::ostream& os, const file_type type, std::string& err_msg)
+  {
+  arma_extra_debug_sigprint();
+  
+  switch(type)
+    {
+    case arma_binary:
+      return diskio::save_arma_binary(x, os);
+      break;
+      
+    case ppm_binary:
+      return diskio::save_ppm_binary(x, os);
+      break;
+    
+    default:
+      err_msg = " [unsupported type] filename = ";
+      return false;
+    }
+  }
+
+
+
+template<typename eT>
+inline
+bool
+field_aux::load(field< Col<eT> >& x, const std::string& name, const file_type type, std::string& err_msg)
   {
   arma_extra_debug_sigprint();
   
   switch(type)
     {
     case auto_detect:
-      diskio::load_auto_detect(x, name);
+      return diskio::load_auto_detect(x, name, err_msg);
       break;
     
     case arma_binary:
-      diskio::load_arma_binary(x, name);
+      return diskio::load_arma_binary(x, name, err_msg);
       break;
       
     case ppm_binary:
-      diskio::load_ppm_binary(x, name);
+      return diskio::load_ppm_binary(x, name, err_msg);
       break;
     
     default:
-      arma_stop("field_aux::load(): unsupported type");
+      err_msg = " [unsupported type] filename = ";
+      return false;
     }
   }
 
@@ -978,74 +1417,293 @@ field_aux::load(field< Row<eT> >& x, const std::string& name, const file_type ty
 
 template<typename eT>
 inline
-void
-field_aux::save(const field< Cube<eT> >& x, const std::string& name, const file_type type)
-  {
-  arma_extra_debug_sigprint();
-  
-  switch(type)
-    {
-    case arma_binary:
-      diskio::save_arma_binary(x, name);
-      break;
-      
-    case ppm_binary:
-      diskio::save_ppm_binary(x, name);
-      break;
-    
-    default:
-      arma_stop("field_aux::save(): unsupported type");
-    }
-  }
-
-
-
-template<typename eT>
-inline
-void
-field_aux::load(field< Cube<eT> >& x, const std::string& name, const file_type type)
+bool
+field_aux::load(field< Col<eT> >& x, std::istream& is, const file_type type, std::string& err_msg)
   {
   arma_extra_debug_sigprint();
   
   switch(type)
     {
     case auto_detect:
-      diskio::load_auto_detect(x, name);
+      return diskio::load_auto_detect(x, is, err_msg);
       break;
     
     case arma_binary:
-      diskio::load_arma_binary(x, name);
+      return diskio::load_arma_binary(x, is, err_msg);
       break;
       
     case ppm_binary:
-      diskio::load_ppm_binary(x, name);
+      return diskio::load_ppm_binary(x, is, err_msg);
       break;
     
     default:
-      arma_stop("field_aux::load(): unsupported type");
+      err_msg = " [unsupported type] filename = ";
+      return false;
+    }
+  }
+
+
+
+template<typename eT>
+inline
+bool
+field_aux::save(const field< Row<eT> >& x, const std::string& name, const file_type type, std::string& err_msg)
+  {
+  arma_extra_debug_sigprint();
+  
+  switch(type)
+    {
+    case arma_binary:
+      return diskio::save_arma_binary(x, name, err_msg);
+      break;
+      
+    case ppm_binary:
+      return diskio::save_ppm_binary(x, name, err_msg);
+      break;
+    
+    default:
+      err_msg = " [unsupported type] filename = ";
+      return false;
+    }
+  }
+
+
+
+template<typename eT>
+inline
+bool
+field_aux::save(const field< Row<eT> >& x, std::ostream& os, const file_type type, std::string& err_msg)
+  {
+  arma_extra_debug_sigprint();
+  
+  switch(type)
+    {
+    case arma_binary:
+      return diskio::save_arma_binary(x, os, err_msg);
+      break;
+      
+    case ppm_binary:
+      return diskio::save_ppm_binary(x, os, err_msg);
+      break;
+    
+    default:
+      err_msg = " [unsupported type] filename = ";
+      return false;
+    }
+  }
+
+
+
+template<typename eT>
+inline
+bool
+field_aux::load(field< Row<eT> >& x, const std::string& name, const file_type type, std::string& err_msg)
+  {
+  arma_extra_debug_sigprint();
+  
+  switch(type)
+    {
+    case auto_detect:
+      return diskio::load_auto_detect(x, name, err_msg);
+      break;
+    
+    case arma_binary:
+      return diskio::load_arma_binary(x, name, err_msg);
+      break;
+      
+    case ppm_binary:
+      return diskio::load_ppm_binary(x, name, err_msg);
+      break;
+    
+    default:
+      err_msg = " [unsupported type] filename = ";
+      return false;
+    }
+  }
+
+
+
+template<typename eT>
+inline
+bool
+field_aux::load(field< Row<eT> >& x, std::istream& is, const file_type type, std::string& err_msg)
+  {
+  arma_extra_debug_sigprint();
+  
+  switch(type)
+    {
+    case auto_detect:
+      return diskio::load_auto_detect(x, is, err_msg);
+      break;
+    
+    case arma_binary:
+      return diskio::load_arma_binary(x, is, err_msg);
+      break;
+      
+    case ppm_binary:
+      return diskio::load_ppm_binary(x, is, err_msg);
+      break;
+    
+    default:
+      err_msg = " [unsupported type] filename = ";
+      return false;
+    }
+  }
+
+
+
+template<typename eT>
+inline
+bool
+field_aux::save(const field< Cube<eT> >& x, const std::string& name, const file_type type, std::string& err_msg)
+  {
+  arma_extra_debug_sigprint();
+  
+  switch(type)
+    {
+    case arma_binary:
+      return diskio::save_arma_binary(x, name, err_msg);
+      break;
+      
+    case ppm_binary:
+      return diskio::save_ppm_binary(x, name, err_msg);
+      break;
+    
+    default:
+      err_msg = " [unsupported type] filename = ";
+      return false;
+    }
+  }
+
+
+
+template<typename eT>
+inline
+bool
+field_aux::save(const field< Cube<eT> >& x, std::ostream& os, const file_type type, std::string& err_msg)
+  {
+  arma_extra_debug_sigprint();
+  
+  switch(type)
+    {
+    case arma_binary:
+      return diskio::save_arma_binary(x, os, err_msg);
+      break;
+      
+    case ppm_binary:
+      return diskio::save_ppm_binary(x, os, err_msg);
+      break;
+    
+    default:
+      err_msg = " [unsupported type] filename = ";
+      return false;
+    }
+  }
+
+
+
+template<typename eT>
+inline
+bool
+field_aux::load(field< Cube<eT> >& x, const std::string& name, const file_type type, std::string& err_msg)
+  {
+  arma_extra_debug_sigprint();
+  
+  switch(type)
+    {
+    case auto_detect:
+      return diskio::load_auto_detect(x, name, err_msg);
+      break;
+    
+    case arma_binary:
+      return diskio::load_arma_binary(x, name, err_msg);
+      break;
+      
+    case ppm_binary:
+      return diskio::load_ppm_binary(x, name, err_msg);
+      break;
+    
+    default:
+      err_msg = " [unsupported type] filename = ";
+      return false;
+    }
+  }
+
+
+
+template<typename eT>
+inline
+bool
+field_aux::load(field< Cube<eT> >& x, std::istream& is, const file_type type, std::string& err_msg)
+  {
+  arma_extra_debug_sigprint();
+  
+  switch(type)
+    {
+    case auto_detect:
+      return diskio::load_auto_detect(x, is, err_msg);
+      break;
+    
+    case arma_binary:
+      return diskio::load_arma_binary(x, is, err_msg);
+      break;
+      
+    case ppm_binary:
+      return diskio::load_ppm_binary(x, is, err_msg);
+      break;
+    
+    default:
+      err_msg = " [unsupported type] filename = ";
+      return false;
     }
   }
 
 
 
 inline
-void
-field_aux::save(const field< std::string >& x, const std::string& name, const file_type type)
+bool
+field_aux::save(const field< std::string >& x, const std::string& name, const file_type type, std::string& err_msg)
   {
   arma_extra_debug_sigprint();
   
-  diskio::save_std_string(x, name);
+  err_msg.clear();
+  
+  return diskio::save_std_string(x, name);
   }
 
 
 
 inline
-void
-field_aux::load(field< std::string >& x, const std::string& name, const file_type type)
+bool
+field_aux::save(const field< std::string >& x, std::ostream& os, const file_type type, std::string& err_msg)
   {
   arma_extra_debug_sigprint();
   
-  diskio::load_std_string(x, name);
+  err_msg.clear();
+  
+  return diskio::save_std_string(x, os);
+  }
+
+
+
+inline
+bool
+field_aux::load(field< std::string >& x, const std::string& name, const file_type type, std::string& err_msg)
+  {
+  arma_extra_debug_sigprint();
+  
+  return diskio::load_std_string(x, name, err_msg);
+  }
+
+
+
+inline
+bool
+field_aux::load(field< std::string >& x, std::istream& is, const file_type type, std::string& err_msg)
+  {
+  arma_extra_debug_sigprint();
+  
+  return diskio::load_std_string(x, is, err_msg);
   }
 
 
