@@ -1,4 +1,5 @@
-// Copyright (C) 2009 NICTA
+// Copyright (C) 2010 NICTA and the authors listed below
+// http://nicta.com.au
 // 
 // Authors:
 // - Conrad Sanderson (conradsand at ieee dot org)
@@ -18,9 +19,14 @@
 
 
 template<typename T1, typename T2>
-inline
+arma_inline
+arma_warn_unused
 typename T1::elem_type
-dot(const Base<typename T1::elem_type,T1>& A, const Base<typename T1::elem_type,T2>& B)
+dot
+  (
+  const Base<typename T1::elem_type,T1>& A,
+  const Base<typename T1::elem_type,T2>& B
+  )
   {
   arma_extra_debug_sigprint();
   
@@ -30,9 +36,15 @@ dot(const Base<typename T1::elem_type,T1>& A, const Base<typename T1::elem_type,
 
 
 template<typename T1, typename T2>
-inline
+arma_inline
+arma_warn_unused
 typename T1::elem_type
-norm_dot(const Base<typename T1::elem_type,T1>& A, const Base<typename T1::elem_type,T2>& B)
+norm_dot
+  (
+  const Base<typename T1::elem_type,T1>& A, 
+  const Base<typename T1::elem_type,T2>& B,
+  const typename arma_blas_type_only<typename T1::elem_type>::result* junk = 0
+  )
   {
   arma_extra_debug_sigprint();
   
