@@ -1,4 +1,5 @@
-// Copyright (C) 2009 NICTA
+// Copyright (C) 2010 NICTA and the authors listed below
+// http://nicta.com.au
 // 
 // Authors:
 // - Conrad Sanderson (conradsand at ieee dot org)
@@ -16,11 +17,13 @@
 //! \addtogroup fn_inv
 //! @{
 
+
+
 //! delayed matrix inverse
 template<typename T1>
-inline
+arma_inline
 const Op<T1, op_inv>
-inv(const Base<typename T1::elem_type,T1>& X)
+inv(const Base<typename T1::elem_type,T1>& X, const typename arma_blas_type_only<typename T1::elem_type>::result* junk = 0)
   {
   arma_extra_debug_sigprint();
   
@@ -31,7 +34,7 @@ inv(const Base<typename T1::elem_type,T1>& X)
 
 //! remove the inverse operation if applied twice consecutively
 template<typename T1>
-inline
+arma_inline
 const T1&
 inv(const Op<T1, op_inv>& X)
   {
@@ -40,4 +43,6 @@ inv(const Op<T1, op_inv>& X)
   return X.m;
   }
 
+
+
 //! @}
