@@ -1,4 +1,5 @@
-// Copyright (C) 2009 NICTA
+// Copyright (C) 2010 NICTA and the authors listed below
+// http://nicta.com.au
 // 
 // Authors:
 // - Conrad Sanderson (conradsand at ieee dot org)
@@ -19,24 +20,36 @@
 
 //! find the variance of an array
 template<typename eT>
-inline 
+inline
 eT
 op_var::direct_var(const eT* const X, const u32 n_elem, const u32 norm_type)
   {
   arma_extra_debug_sigprint();
   
   eT acc1 = eT(0);
-  eT acc2 = eT(0);
   
   for(u32 i=0; i<n_elem; ++i)
     {
-    const eT tmp_val = X[i];
-    acc1 += tmp_val;
-    acc2 += tmp_val*tmp_val;
+    acc1 += X[i];
     }
   
+  const eT div_val = (n_elem > 0) ? eT(n_elem) : eT(1);
+  acc1 /= div_val;
+  
+  eT acc2 = eT(0);
+  eT acc3 = eT(0);
+
+  for(u32 i=0; i<n_elem; ++i)
+    {
+    const eT tmp = acc1 - X[i];
+  
+    acc2 += tmp*tmp;
+    acc3 += tmp;
+    }
+  
+  
   const eT norm_val = (norm_type == 0) ? ( (n_elem > 1) ? eT(n_elem-1) : eT(1) ) : eT(n_elem);
-  const eT var_val  = (acc2 - acc1*acc1/eT(n_elem)) / norm_val;
+  const eT var_val  = (acc2 - acc3*acc3/div_val) / norm_val;
   
   return var_val;
   }
@@ -45,7 +58,7 @@ op_var::direct_var(const eT* const X, const u32 n_elem, const u32 norm_type)
 
 //! find the variance of an array (version for complex numbers)
 template<typename T>
-inline 
+inline
 T
 op_var::direct_var(const std::complex<T>* const X, const u32 n_elem, const u32 norm_type)
   {
@@ -54,22 +67,95 @@ op_var::direct_var(const std::complex<T>* const X, const u32 n_elem, const u32 n
   typedef typename std::complex<T> eT;
   
   eT acc1 = eT(0);
-  T  acc2 = T(0);
   
   for(u32 i=0; i<n_elem; ++i)
     {
     acc1 += X[i];
-    acc2 += std::norm(X[i]);
+    }
+  
+  const T div_val = (n_elem > 0) ? T(n_elem) : T(1);
+  acc1 /= div_val;
+  
+  T  acc2 =  T(0);
+  eT acc3 = eT(0);
+  
+  for(u32 i=0; i<n_elem; ++i)
+    {
+    const eT tmp = acc1 - X[i];
+    
+    acc2 += std::norm(tmp);
+    acc3 += tmp;
     }
   
   const T norm_val = (norm_type == 0) ? ( (n_elem > 1) ? T(n_elem-1) : T(1) ) : T(n_elem);
-  const T var_val  = (acc2 - std::norm(acc1)/T(n_elem)) / norm_val;
+  const T var_val  = (acc2 - std::norm(acc3)/div_val) / norm_val;
   
   return var_val;
   }
 
 
 
+//! find the variance of a subview_row
+template<typename eT>
+inline 
+typename get_pod_type<eT>::result
+op_var::direct_var(const subview_row<eT>& X, const u32 norm_type)
+  {
+  arma_extra_debug_sigprint();
+  
+  const u32 n_elem = X.n_elem;
+  
+  podarray<eT> tmp(n_elem);
+  
+  eT* tmp_mem = tmp.memptr();
+  
+  for(u32 i=0; i<n_elem; ++i)
+    {
+    tmp_mem[i] = X[i];
+    }
+  
+  return op_var::direct_var(tmp_mem, n_elem, norm_type);
+  }
+
+
+
+//! find the variance of a subview_col
+template<typename eT>
+inline 
+typename get_pod_type<eT>::result
+op_var::direct_var(const subview_col<eT>& X, const u32 norm_type)
+  {
+  arma_extra_debug_sigprint();
+  
+  return op_var::direct_var(X.colptr(0), X.n_elem, norm_type);
+  }
+
+
+
+//! find the variance of a diagview
+template<typename eT>
+inline 
+typename get_pod_type<eT>::result
+op_var::direct_var(const diagview<eT>& X, const u32 norm_type)
+  {
+  arma_extra_debug_sigprint();
+  
+  const u32 n_elem = X.n_elem;
+  
+  podarray<eT> tmp(n_elem);
+  
+  eT* tmp_mem = tmp.memptr();
+  
+  for(u32 i=0; i<n_elem; ++i)
+    {
+    tmp_mem[i] = X[i];
+    }
+  
+  return op_var::direct_var(tmp_mem, n_elem, norm_type);
+  }
+
+
+
 //! \brief
 //! For each row or for each column, find the variance.
 //! The result is stored in a dense matrix that has either one column or one row.
@@ -77,7 +163,7 @@ op_var::direct_var(const std::complex<T>* const X, const u32 n_elem, const u32 n
 template<typename eT>
 inline
 void
-op_var::apply(Mat<eT>& out, const Mat<eT>& X, const u32 norm_type, const u32 dim)
+op_var::apply(Mat< typename get_pod_type<eT>::result >& out, const Mat<eT>& X, const u32 norm_type, const u32 dim)
   {
   arma_extra_debug_sigprint();
   
@@ -103,25 +189,23 @@ op_var::apply(Mat<eT>& out, const Mat<eT>& X, const u32 norm_type, const u32 dim
     {
     arma_extra_debug_print("op_var::apply(), dim = 1");
     
-    out.set_size(X.n_rows, 1);
+    const u32 n_rows = X.n_rows;
+    const u32 n_cols = X.n_cols;
     
-    const eT norm_val = (norm_type == 0) ? ( (X.n_cols > 1) ? eT(X.n_cols-1) : eT(1) ) : eT(X.n_cols);
+    out.set_size(n_rows, 1);
     
-    for(u32 row=0; row<X.n_rows; ++row)
+    podarray<eT> tmp(n_cols);
+    
+    eT* tmp_mem = tmp.memptr();
+    
+    for(u32 row=0; row<n_rows; ++row)
       {
-      eT acc1 = eT(0);
-      eT acc2 = eT(0);
-  
-      for(u32 col=0; col<X.n_cols; ++col)
+      for(u32 col=0; col<n_cols; ++col)
         {
-        const eT tmp_val = X.at(row,col);
-        acc1 += tmp_val;
-        acc2 += tmp_val*tmp_val;
+        tmp_mem[col] = X.at(row,col);
         }
       
-      const eT var_val = (acc2 - acc1*acc1/eT(X.n_cols)) / norm_val;
-      
-      out[row] = var_val;
+      out[row] = op_var::direct_var(tmp_mem, n_cols, norm_type);
       }
     
     }
@@ -130,176 +214,4 @@ op_var::apply(Mat<eT>& out, const Mat<eT>& X, const u32 norm_type, const u32 dim
 
 
 
-//! implementation for complex numbers
-template<typename T>
-inline
-void
-op_var::apply(Mat<T>& out, const Mat< std::complex<T> >& X, const u32 norm_type, const u32 dim)
-  {
-  arma_extra_debug_sigprint();
-  
-  typedef typename std::complex<T> eT;
-  
-  arma_debug_check( (X.n_elem == 0), "var(): given matrix has no elements" );
-  
-  arma_debug_check( (norm_type > 1), "var(): incorrect usage. norm_type must be 0 or 1");
-  arma_debug_check( (dim > 1),       "var(): incorrect usage. dim must be 0 or 1"      );
-  
-  
-  if(dim == 0)
-    {
-    arma_extra_debug_print("op_var::apply(), dim = 0");
-    
-    out.set_size(1, X.n_cols);
-    
-    for(u32 col=0; col<X.n_cols; ++col)
-      {
-      out[col] = op_var::direct_var( X.colptr(col), X.n_rows, norm_type );
-      }
-    }
-  else
-  if(dim == 1)
-    {
-    arma_extra_debug_print("op_var::apply(), dim = 1");
-    
-    out.set_size(X.n_rows, 1);
-    
-    const T norm_val = (norm_type == 0) ? ( (X.n_cols > 1) ? T(X.n_cols-1) : T(1) ) : T(X.n_cols);
-    
-    for(u32 row=0; row<X.n_rows; ++row)
-      {
-      eT acc1 = eT(0);
-      T  acc2 = T(0);
-  
-      for(u32 col=0; col<X.n_cols; ++col)
-        {
-        acc1 += X.at(row,col);
-        acc2 += std::norm(X.at(row,col));
-        }
-      
-      const T var_val = (acc2 - std::norm(acc1)/T(X.n_cols)) / norm_val;
-      
-      out[row] = var_val;
-      }
-    
-    }
-  
-  }
-
-
-
-//! find the variance of a subview
-template<typename eT>
-inline 
-eT
-op_var::direct_var(const subview<eT>& X, const u32 norm_type)
-  {
-  arma_extra_debug_sigprint();
-  
-  const u32 n_elem = X.n_elem;
-  
-  eT acc1 = eT(0);
-  eT acc2 = eT(0);
-  
-  for(u32 i=0; i<n_elem; ++i)
-    {
-    const eT tmp_val = X[i];
-    acc1 += tmp_val;
-    acc2 += tmp_val*tmp_val;
-    }
-  
-  const eT norm_val = (norm_type == 0) ? ( (n_elem > 1) ? eT(n_elem-1) : eT(1) ) : eT(n_elem);
-  const eT var_val  = (acc2 - acc1*acc1/eT(n_elem)) / norm_val;
-  
-  return var_val;
-  }
-
-
-
-//! find the variance of a subview (version for complex numbers)
-template<typename T>
-inline 
-T
-op_var::direct_var(const subview< std::complex<T> >& X, const u32 norm_type)
-  {
-  arma_extra_debug_sigprint();
-  
-  typedef typename std::complex<T> eT;
-  
-  const u32 n_elem = X.n_elem;
-  
-  eT acc1 = eT(0);
-  T  acc2 = T(0);
-  
-  for(u32 i=0; i<n_elem; ++i)
-    {
-    acc1 += X[i];
-    acc2 += std::norm(X[i]);
-    }
-  
-  const T norm_val = (norm_type == 0) ? ( (n_elem > 1) ? T(n_elem-1) : T(1) ) : T(n_elem);
-  const T var_val  = (acc2 - std::norm(acc1)/T(n_elem)) / norm_val;
-  
-  return var_val;
-  }
-
-
-
-//! find the variance of a diagview
-template<typename eT>
-inline 
-eT
-op_var::direct_var(const diagview<eT>& X, const u32 norm_type)
-  {
-  arma_extra_debug_sigprint();
-  
-  const u32 n_elem = X.n_elem;
-  
-  eT acc1 = eT(0);
-  eT acc2 = eT(0);
-  
-  for(u32 i=0; i<n_elem; ++i)
-    {
-    const eT tmp_val = X[i];
-    acc1 += tmp_val;
-    acc2 += tmp_val*tmp_val;
-    }
-  
-  const eT norm_val = (norm_type == 0) ? ( (n_elem > 1) ? eT(n_elem-1) : eT(1) ) : eT(n_elem);
-  const eT var_val  = (acc2 - acc1*acc1/eT(n_elem)) / norm_val;
-  
-  return var_val;
-  }
-
-
-
-//! find the variance of a diagview (version for complex numbers)
-template<typename T>
-inline 
-T
-op_var::direct_var(const diagview< std::complex<T> >& X, const u32 norm_type)
-  {
-  arma_extra_debug_sigprint();
-  
-  typedef typename std::complex<T> eT;
-  
-  const u32 n_elem = X.n_elem;
-  
-  eT acc1 = eT(0);
-  T  acc2 = T(0);
-  
-  for(u32 i=0; i<n_elem; ++i)
-    {
-    acc1 += X[i];
-    acc2 += std::norm(X[i]);
-    }
-  
-  const T norm_val = (norm_type == 0) ? ( (n_elem > 1) ? T(n_elem-1) : T(1) ) : T(n_elem);
-  const T var_val  = (acc2 - std::norm(acc1)/T(n_elem)) / norm_val;
-  
-  return var_val;
-  }
-
-
-
 //! @}
