@@ -1,4 +1,5 @@
-// Copyright (C) 2009 NICTA
+// Copyright (C) 2010 NICTA and the authors listed below
+// http://nicta.com.au
 // 
 // Authors:
 // - Conrad Sanderson (conradsand at ieee dot org)
@@ -30,27 +31,18 @@ class op_var
   inline static T direct_var(const std::complex<T>* const X, const u32 N, const u32 norm_type = 0);
   
   
+  template<typename eT>
+  inline static typename get_pod_type<eT>::result direct_var(const subview_row<eT>& X, const u32 norm_type = 0);
   
   template<typename eT>
-  inline static void apply(Mat<eT>& out, const Mat<eT>& X, const u32 norm_type, const u32 dim);
-  
-  template<typename T>
-  inline static void apply(Mat<T>& out, const Mat< std::complex<T> >& X, const u32 norm_type, const u32 dim);
+  inline static typename get_pod_type<eT>::result direct_var(const subview_col<eT>& X, const u32 norm_type = 0);
   
+  template<typename eT>
+  inline static typename get_pod_type<eT>::result direct_var(const diagview<eT>& X, const u32 norm_type = 0);
   
   
   template<typename eT>
-  inline static eT direct_var(const subview<eT>& X, const u32 norm_type = 0);
-  
-  template<typename T>
-  inline static T direct_var(const subview< std::complex<T> >& X, const u32 norm_type = 0);
-  
-  
-  template<typename eT>
-  inline static eT direct_var(const diagview<eT>& X, const u32 norm_type = 0);
-  
-  template<typename T>
-  inline static T direct_var(const diagview< std::complex<T> >& X, const u32 norm_type = 0);
+  inline static void apply(Mat< typename get_pod_type<eT>::result >& out, const Mat<eT>& X, const u32 norm_type, const u32 dim);
   
   };
 
